#pragma once

#include <cstdint>
#include <vector>
#include <limits>

typedef uint32_t letter_t;
constexpr letter_t MIN_LETTER = std::numeric_limits<letter_t>::min();
constexpr letter_t MAX_LETTER = std::numeric_limits<letter_t>::max();

enum Player : int8_t {
    SYS_PLAYER = 1,
    ENV_PLAYER = -1,
    UNKNOWN = 0,
};

enum Parity { EVEN = 0, ODD = 1 };

typedef uint32_t node_id_t;
typedef uint32_t edge_id_t;
typedef uint32_t color_t;

static_assert(sizeof(node_id_t) == sizeof(color_t), "sizes of node type and color type do not match");

constexpr node_id_t NODE_BOTTOM = std::numeric_limits<node_id_t>::max() - 1;
constexpr node_id_t NODE_TOP = std::numeric_limits<node_id_t>::max() - 2;
constexpr node_id_t NODE_NONE = std::numeric_limits<node_id_t>::max() - 3;
constexpr edge_id_t EDGE_BOTTOM = std::numeric_limits<edge_id_t>::max();

typedef std::vector<node_id_t> product_state_t;

struct Edge {
    const node_id_t successor;
    const color_t color;

    Edge(const node_id_t successor, const color_t color) : successor(successor), color(color) {};
};
