#include "Parser.h"

#include <iostream>
#include <fstream>
#include <sstream>

namespace ltl {

Parser::Parser(const owl::OwlThread& owl) : owl(owl) { }

Parser::~Parser() { }

std::string Parser::parse_file(const std::string& input_file) {
    // read input file
    std::string input_string;

    std::ifstream infile(input_file);
    if (infile.is_open()) {
        input_string = std::string((std::istreambuf_iterator<char>(infile)), std::istreambuf_iterator<char>());
    }
    else {
        throw std::runtime_error("could not open input file");
    }
    if (infile.bad()) {
        throw std::runtime_error("could not read input file");
    }
    infile.close();

    return input_string;
}

owl::Formula Parser::parse_string(const std::string& input_string, std::vector<std::string>& inputs, std::vector<std::string>& outputs, const bool print_formula) const {
    if (print_formula) {
        std::cout << "Input: " << std::endl << input_string << std::endl;
    }
    owl::FormulaFactory factory = owl.createFormulaFactory();
    owl::Formula formula = parse_string_with_factory(factory, input_string, inputs, outputs);
    if (print_formula) {
        std::cout << "Parsed formula:" << std::endl;
        formula.print();
        std::cout << "Inputs:";
        for (const auto& i : inputs) {
            std::cout << " " << i;
        }
        std::cout << std::endl;
        std::cout << "Outputs:";
        for (const auto& o : outputs) {
            std::cout << " " << o;
        }
        std::cout << std::endl;
    }

    return formula;
}

}
